from bs4 import BeautifulSoup
import requests

def getCompanyInfo(ttName):
    """Returns company info of the ttName"""
    res = requests.get("https://www.tickertape.in/stocks/{}".format(ttName))
    soup = BeautifulSoup(res.content, "html.parser")
    ticker = soup.find("span", {"class":"ticker"}).text
    name = soup.find("h1", {"class":"security-name"}).text
    price = soup.find("span", {"class":"current-price"}).text
    return {
        "ttName": ttName,
        "name": name,
        "ticker": ticker,
        "price": price
    }

def getTopCompanies():
    """Returns the list of ttName of top companies"""
    res = requests.get("https://www.tickertape.in/stocks?filter=top")
    soup = BeautifulSoup(res.content, "html.parser")
    
    pageDiv = soup.find("div", {"class":"page"})
    lis = pageDiv.find_all("li")
    
    topCompanies = []
    for li in lis:
        ttName = li.find("a").get("href").split("/stocks/")[-1]
        topCompanies.append(ttName)
    return topCompanies

def getAllCompanies():
    """Returns the list of ttName of all companies"""
    res = requests.get("https://www.tickertape.in/stocks")
    soup = BeautifulSoup(res.content, "html.parser")
    links_div = soup.find("div", {"class":"link-list"})
    alinks = links_div.find_all("a")
    
    allCompanies = []
    temp_names = []
    for alink in alinks:
        ares = requests.get("https://www.tickertape.in/stocks?filter={}".format(alink.text))
        asoup = BeautifulSoup(ares.content, "html.parser")
        
        pageDiv = asoup.find("div", {"class":"page"})
        if not pageDiv:
            continue
        lis = pageDiv.find_all("li")
    
        for li in lis:
            ttName = li.find("a").get("href").split("/stocks/")[-1]
            if not ttName in temp_names:
                try:
                    temp_names.append(ttName)
                    allCompanies.append(ttName)
                except:
                    continue
    return allCompanies


import matplotlib.pyplot as plt

from . import plot_colors as plotColors

def trendPlot(xValues, yValues, **kwargs):
    """Plots matplotlib for given values

    Parameters
    ---
    xValues: list
        List of integers on x-axis
    
    yValues: list
        List of integer on y-axis

    **kwargs:
        * xTicks: list
            Data ticks for x values, defaults to xValues

        * rotateXTicks int
            Value is passed as matlplotlib xTicks rotation

        * xLabel: list
            Data label for x values
        
        * yLabel: list
            Data label for y values

        * plotTitle
            Plot title

        * trendValues
            List of integers to plot trend line

        * trendColor
            Color of the trend line

        * legends
            Legends for the plot

        * text
            Text to write in the center of the plot

        * textBackground
            Background color of the text

        * watermark
            Watermark for the plot

        * showValues
            Show bar values

        * saveToFile
            File path to save the image
    """

    midX = min(xValues) + (max(xValues) - min(xValues)) / 2
    midY = max(yValues) / 2

    fix, ax = plt.subplots(figsize=(10, 5))
    ax.bar(xValues, yValues, color = plotColors.YELLOW, width = 0.6)

    # Keyword arguments
    # xTicks, rotateXTicks
    rotateXTicks = 0
    if "rotateXTicks" in kwargs:
        rotateXTicks = kwargs["rotateXTicks"]

    plt.xticks(xValues, rotation = rotateXTicks)
    if "xTicks" in kwargs:
        ax.set_xticklabels(kwargs["xTicks"], rotation = rotateXTicks)

    # xLabel
    if "xLabel" in kwargs:
        ax.set_xlabel(kwargs["xLabel"], fontsize = 14)

    # yLabel
    if "yLabel" in kwargs:
        ax.set_ylabel(kwargs["yLabel"], fontsize = 14)

    # plotTitle
    if "plotTitle" in kwargs:
        ax.set_title(kwargs["plotTitle"], fontsize = 18)

    # trendValues, trendColor
    if "trendValues" in kwargs:
        trendValues = kwargs["trendValues"]
        trendArgs = dict(lw = 2.5)
        if "trendColor" in kwargs:
            trendArgs["c"] = kwargs["trendColor"]
        ax.plot(xValues, trendValues, **trendArgs)

    # legends
    if "legends" in kwargs:
        legends = kwargs["legends"]
        plt.legend(legends, loc ="upper left")

    # text, textBackground
    if "text" in kwargs:
        text = kwargs["text"]
        
        textBackground = "#f7f7f7"
        if "textBackground" in kwargs:
            textBackground = kwargs["textBackground"]

        textProps = dict(boxstyle='round', facecolor=textBackground, alpha=0.25)
        plt.text(
            midX, midY, text, fontsize=14, 
            bbox=textProps, va="center", ha="center"
        )

    # watermark
    if "watermark" in kwargs:
        watermark = kwargs["watermark"]
        plt.text(
            midX, midY / 10, watermark, fontsize=8, 
            bbox=dict(
                boxstyle="round", 
                facecolor=plotColors.LIGHT_GREY, 
                alpha=0.1
            ),
            va="center", ha="center", color="grey"
        )

    if "showValues" in kwargs:
        if kwargs["showValues"]:
            for i, bar in enumerate(ax.patches):
                ax.text(
                    bar.get_x() + bar.get_width() / 2,
                    bar.get_height() / 2 + bar.get_y(),
                    yValues[i], ha = 'center',
                    color = "black", size = 10
                )

    if "saveToFile" in kwargs:
        fileName = kwargs["saveToFile"]
        plt.savefig(fileName, bbox_inches="tight", dpi=150)

    plt.show()


import requests
import json

statementTypes = dict(
    income = "income",
    balancesheet = "balancesheet",
    cashflow = "cashflow"
)

def getStatement(ttName, statementType):
    """Returns records of the requested statement"""
    ttID = ttName.split("-")[-1]
    statement = dict()
    url_string = "https://api.tickertape.in/stocks/financials/{}/{}/annual/normal?count=10"
    url = url_string.format(statementType, ttID)
    records = json.loads(requests.get(url).content)["data"]
    for record in records:
        if record["displayPeriod"] == "TTM":
            continue
        
        for key in record:
            try:
                record[key] = round(record[key], 2)
            except:
                pass
        statement[record["displayPeriod"].split("FY")[1].strip()] = record
    return statement

def getYearsAndValues(statement, key):
    "Returns yearly values of the requested key"
    yearsAndValues = {}
    for fy in sorted(statement.keys()):
        year = fy.split("FY")[-1].strip()
        value = statement[fy][key]
        if not value:
            value = 0
        yearsAndValues[year] = value
    return yearsAndValues



from sklearn.linear_model import LinearRegression

from .utils import npArrayReshape

class QpyLinearRegression:
    """Wrapper over sklearn LinearRegression"""
    def __init__(self, xTrain, yTrain):
        self.xTrain = npArrayReshape(xTrain)
        self.yTrain = npArrayReshape(yTrain)
        self.model = LinearRegression()

    def train(self):
        self.model.fit(self.xTrain, self.yTrain)

    def getPrediction(self, xPredict = None):
        if not xPredict:
            xPredict = self.xTrain
        else: xPredict = npArrayReshape(xPredict)

        yPredict = [y[0] for y in self.model.predict(xPredict).tolist()]
        return yPredict


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name='qwertypy',
    version='0.5.4',
    author='Vipul Sharma | @qwertyvipul',
    description='My personal python utilities library.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/qwertyvipul/qwertypy",
    project_urls={
        "Repository": "https://github.com/qwertyvipul/qwertypy",
        "Changelog": "https://github.com/qwertyvipul/qwertypy/blob/main/CHANGELOG.md"
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires='>=3.6',
    test_suite='nose.collector',
    tests_require=['nose'],
)